@@ -256,6 +256,7 @@ class Makefile:
         self._debug = debug
         self._makefile = makefile
         self._installs = installs
+        self._infix = ""
 
         # Make sure the destination directory is an absolute path.
         if dir:
@@ -418,10 +419,18 @@ class Makefile:
         rpaths = _UniqueList()
 
         for l in self.extra_lib_dirs:
+            l_dir = os.path.dirname(l)
+
+            # This is a hack to ignore PyQt's internal support libraries.
+            if '/qpy/' in l_dir:
+                continue
+
             # Ignore relative directories.  This is really a hack to handle
             # SIP v3 inter-module linking.
-            if os.path.dirname(l) not in ("", ".", ".."):
-                rpaths.append(l)
+            if l_dir in ("", ".", ".."):
+                continue
+
+            rpaths.append(l)
 
         if self._python:
             incdir.append(self.config.py_inc_dir)
@@ -539,6 +548,14 @@ class Makefile:
             lflags.extend(self.optional_list("LFLAGS_THREAD"))
 
         if self._qt:
+            # Get the name of the mkspecs directory.
+            try:
+                specd_base = self.config.qt_data_dir
+            except AttributeError:
+                specd_base = self.config.qt_dir
+
+            mkspecs = os.path.join(specd_base, "mkspecs")
+
             if self.generator != "UNIX" and win_shared:
                 defines.append("QT_DLL")
 
@@ -585,6 +602,10 @@ class Makefile:
             rpaths.extend(libdir_qt)
 
             if self.config.qt_version >= 0x040000:
+                # Try and read QT_LIBINFIX from qconfig.pri.
+                qconfig = os.path.join(mkspecs, "qconfig.pri")
+                self._infix = self._extract_value(qconfig, "QT_LIBINFIX")
+
                 # For Windows: the macros that define the dependencies on
                 # Windows libraries.
                 wdepmap = {
@@ -676,15 +697,10 @@ class Makefile:
                 libs.extend(self._dependent_libs(self.config.qt_lib))
 
             # Handle header directories.
-            try:
-                specd_base = self.config.qt_data_dir
-            except AttributeError:
-                specd_base = self.config.qt_dir
-
-            specd = os.path.join(specd_base, "mkspecs", "default")
+            specd = os.path.join(mkspecs, "default")
 
             if not os.access(specd, os.F_OK):
-                specd = os.path.join(specd_base, "mkspecs", self.config.platform)
+                specd = os.path.join(mkspecs, self.config.platform)
 
             incdir.append(specd)
 
@@ -770,10 +786,15 @@ class Makefile:
         else:
             lib = mname
 
+        lib += self._infix
+
         if self._debug:
             if sys.platform == "win32":
                 lib = lib + "d"
-            elif self.config.qt_version < 0x040200 or sys.platform == "darwin":
+            elif sys.platform == "darwin":
+                if not self._is_framework(mname):
+                    lib = lib + "_debug"
+            elif self.config.qt_version < 0x040200:
                 lib = lib + "_debug"
 
         if sys.platform == "win32" and "shared" in self.config.qt_winconfig.split():
@@ -859,8 +880,6 @@ class Makefile:
         clib is the library name in cannonical form.
         framework is set of the library is implemented as a MacOS framework.
         """
-        prl_libs = []
-
         if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".prl")
         elif sys.platform == "darwin" and framework:
@@ -868,27 +887,36 @@ class Makefile:
         else:
             prl_name = os.path.join(self.config.qt_lib_dir, "lib" + clib + ".prl")
 
-        if os.access(prl_name, os.F_OK):
+        return self._extract_value(prl_name, "QMAKE_PRL_LIBS").split()
+
+    def _extract_value(self, fname, vname):
+        """Return the stripped value from a name=value line in a file.
+
+        fname is the name of the file.
+        vname is the name of the value.
+        """
+        value = ""
+
+        if os.access(fname, os.F_OK):
             try:
-                f = open(prl_name, "r")
+                f = open(fname, "r")
             except IOError:
-                error("Unable to open \"%s\"" % prl_name)
+                error("Unable to open \"%s\"" % fname)
 
             line = f.readline()
             while line:
                 line = line.strip()
                 if line and line[0] != "#":
                     eq = line.find("=")
-                    if eq > 0 and line[:eq].strip() == "QMAKE_PRL_LIBS":
-                        prl_libs = line[eq + 1:].split()
+                    if eq > 0 and line[:eq].strip() == vname:
+                        value = line[eq + 1:].strip()
                         break
 
                 line = f.readline()
 
             f.close()
 
-        return prl_libs
-
+        return value
 
     def parse_build_file(self, filename):
         """
@@ -2340,7 +2368,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                     if orig_rhs is not None:
                         rhs = orig_rhs + " " + rhs
 
-                raw[lhs] = rhs
+                raw[lhs] = _expand_macro_value(raw, rhs, properties)
 
         line = f.readline()
 
@@ -2356,7 +2384,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
     for lhs in list(raw.keys()):
         # Strip any prefix.
-        if lhs.find(macro_prefix) == 0:
+        if lhs.startswith(macro_prefix):
             reflhs = lhs[len(macro_prefix):]
         else:
             reflhs = lhs
@@ -2367,48 +2395,6 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
         rhs = raw[lhs]
 
-        # Resolve any references.
-        estart = rhs.find("$$(")
-        mstart = rhs.find("$$")
-
-        while mstart >= 0 and mstart != estart:
-            rstart = mstart + 2
-            if rstart < len(rhs) and rhs[rstart] == "{":
-                rstart = rstart + 1
-                term = "}"
-            elif rstart < len(rhs) and rhs[rstart] == "[":
-                rstart = rstart + 1
-                term = "]"
-            else:
-                term = string.whitespace
-
-            mend = rstart
-            while mend < len(rhs) and rhs[mend] not in term:
-                mend = mend + 1
-
-            lhs = rhs[rstart:mend]
-
-            if term in "}]":
-                mend = mend + 1
-
-            if term == "]":
-                if properties is None or lhs not in list(properties.keys()):
-                    error("%s: property '%s' is not defined." % (filename, lhs))
-
-                value = properties[lhs]
-            else:
-                try:
-                    value = raw[lhs]
-                except KeyError:
-                    # We used to treat this as an error, but Qt v4.3.0 has at
-                    # least one case that refers to an undefined macro.  If
-                    # qmake handles it then this must be the correct behaviour.
-                    value = ""
-
-            rhs = rhs[:mstart] + value + rhs[mend:]
-            estart = rhs.find("$$(")
-            mstart = rhs.find("$$")
-
         # Expand any POSIX style environment variables.
         pleadin = ["$$(", "$("]
 
@@ -2489,6 +2475,50 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
     return refined
 
 
+def _expand_macro_value(macros, rhs, properties):
+    """Expand the value of a macro based on ones seen so far."""
+    estart = rhs.find("$$(")
+    mstart = rhs.find("$$")
+
+    while mstart >= 0 and mstart != estart:
+        rstart = mstart + 2
+        if rstart < len(rhs) and rhs[rstart] == "{":
+            rstart = rstart + 1
+            term = "}"
+        elif rstart < len(rhs) and rhs[rstart] == "[":
+            rstart = rstart + 1
+            term = "]"
+        else:
+            term = string.whitespace
+
+        mend = rstart
+        while mend < len(rhs) and rhs[mend] not in term:
+            mend = mend + 1
+
+        lhs = rhs[rstart:mend]
+
+        if term in "}]":
+            mend = mend + 1
+
+        if term == "]":
+            # Assume a missing property expands to an empty string.
+            if properties is None:
+                value = ""
+            else:
+                value = properties.get(lhs, "")
+        else:
+            # We used to treat a missing value as an error, but Qt v4.3.0 has
+            # at least one case that refers to an undefined macro.  If qmake
+            # handles it then this must be the correct behaviour.
+            value = macros.get(lhs, "")
+
+        rhs = rhs[:mstart] + value + rhs[mend:]
+        estart = rhs.find("$$(")
+        mstart = rhs.find("$$")
+
+    return rhs
+
+
 def create_wrapper(script, wrapper, gui=0, use_arch=''):
     """Create a platform dependent executable wrapper around a Python script.
 
